@@ -487,6 +487,7 @@ qlist_parse_line(sldns_buffer* buf, char* p)
 	qinfo.qname = sldns_str2wire_dname(nm, &qinfo.qname_len);
 	if(!qinfo.qname)
 		return 0;
+	qinfo.local_alias = NULL;
 	qinfo_query_encode(buf, &qinfo);
 	sldns_buffer_write_u16_at(buf, 0, 0); /* zero ID */
 	if(rec) LDNS_RD_SET(sldns_buffer_begin(buf));
@@ -512,10 +513,12 @@ qlist_grow_capacity(struct perfinfo* info)
 	uint8_t** d = (uint8_t**)calloc(sizeof(uint8_t*), newcap);
 	size_t* l = (size_t*)calloc(sizeof(size_t), newcap);
 	if(!d || !l) fatal_exit("out of memory");
-	memcpy(d, info->qlist_data, sizeof(uint8_t*)*
-		info->qlist_capacity);
-	memcpy(l, info->qlist_len, sizeof(size_t)*
-		info->qlist_capacity);
+	if(info->qlist_data && info->qlist_capacity)
+		memcpy(d, info->qlist_data, sizeof(uint8_t*)*
+			info->qlist_capacity);
+	if(info->qlist_len && info->qlist_capacity)
+		memcpy(l, info->qlist_len, sizeof(size_t)*
+			info->qlist_capacity);
 	free(info->qlist_data);
 	free(info->qlist_len);
 	info->qlist_data = d;
@@ -609,7 +612,7 @@ int main(int argc, char* argv[])
 		case 'd':
 			if(atoi(optarg)==0 && strcmp(optarg, "0")!=0) {
 				printf("-d not a number %s", optarg);
-				return 1;
+				exit(1);
 			}
 			info.duration = atoi(optarg);
 			break;
@@ -634,11 +637,11 @@ int main(int argc, char* argv[])
 	}
 	if(!extstrtoaddr(argv[0], &info.dest, &info.destlen)) {
 		printf("Could not parse ip: %s\n", argv[0]);
-		return 1;
+		exit(1);
 	}
 	if(info.qlist_size == 0) {
 		printf("No queries to make, use -f or -a.\n");
-		return 1;
+		exit(1);
 	}
 	
 	/* do the performance test */
